package com.myoa.engineering.crawl.shopping.dto;

import com.myoa.engineering.crawl.shopping.domain.entity.v1.PpomppuArticle;
import com.myoa.engineering.crawl.shopping.support.dto.SimpleMessageDTO;

import java.time.Instant;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.stream.Collectors;

/**
 * PpomppuArticleTransformer
 *
 * @author Shin Woo-jin (woo-jin.shin@linecorp.com)
 * @since 2021-11-21
 */
public final class PpomppuArticleTransformer {

    private PpomppuArticleTransformer() {
    }

    private static final String MESSAGE_FORMAT_V1 = "%s)) `%s` <%s:LINK>";

    private static final DateTimeFormatter DATE_TIME_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
                                                                                  .withZone(ZoneId.of("Asia/Seoul"));
/*
    public static final Function<PpomppuArticle, SimpleMessageDTO> TRANSFORM_TO_MESSAGE_DTO = entity ->
            SimpleMessageDTO.builder()
                            .requestedAt(Instant.now())
                            .publishedAt(entity.getRegisteredAt())
                            .title(String.format(MESSAGE_FORMAT_V1, entity.getBoardName().getMenuName(), entity.getTitle()))
                            .body(entity.getArticleUrl())
                            .build();
*/
    // https://stackoverflow.com/questions/24882927/using-streams-to-convert-a-list-of-objects-into-a-string-obtained-from-the-tostr
    public static SimpleMessageDTO transform(List<PpomppuArticle> articles) {
        Instant requestedAt = Instant.now();
        String body = articles.stream()
                              .map(PpomppuArticleTransformer::convertToInlineMessage)
                              .collect(Collectors.joining("\n\n"));
        return SimpleMessageDTO.builder()
                               .requestedAt(requestedAt)
                               .title(DATE_TIME_FORMATTER.format(requestedAt))
                               .body(body)
                               .build();
    }

    public static String convertToInlineMessage(PpomppuArticle article) {
        return String.format(MESSAGE_FORMAT_V1,
                             article.getBoardName().getMenuName(), article.getTitle(), article.getArticleUrl());
    }
}
